function [Xn, Fn] = majorw( R, d, W, ftol, gradtol )
%MAJOR Majorisation algorithm for rank reduction of correlation matrices
%  Majorization algorithm for minimization of F(X)=\sum_{i<j}w_{ij}(r_{ij}-<x_i,x_j>)^2 such that diag(X*X')=I with X an nxd matrix with n=size(R,1)=size(R,2). 
%  Convergence is guaranteed if the matrix R has unit diagonal and if all elements of W are positive. 
%  gradtol is the convergence criterion for the norm of the gradient 4*Psi*X with Psi = X*X'-R.
%  ftol is the convergence criterion for the improvement in the function
%  value F. Both criteria should be satisfied for convergence.
%  usage [Xn, Fn] = majorw( R, d, W, ftol, gradtol )

    % first obtain initial guess
    X=guess(R,d);
    
    % dimension
    N=size(R,1);
    
    % sum of weights
    sumW=0.0;
    for i=1:N
        for j=(i+1):N
            sumW = sumW+W(i,j);
        end
    end
    
    % initial gradient, obj value
    mygrad=P_tangent( X, dFw( X, R, W, sumW ) );
    normgrad=norm(mygrad,'fro');
    Fn=Fw(X,R,W,sumW);
    Fold=Fn*(1.0+ftol)+1.0; % at first iteration no Fold is known, this choice guarantees that convergence is not obtained at the first iteration
    
    % output
    n=0;
    disp(sprintf('%s\t%s\t\t%s','iter','|grad|','F(X)'));
	disp(sprintf('%d\t%e\t%e\t%s',n,normgrad,Fn));
    
    % useful in later calculations
    WR=W.*R;

    while (normgrad>gradtol) | ( Fold/Fn-1.0 > ftol)
                
        for i=1:N

            % calculate B and its largest eigenvalue lambda
            B=zeros(d);
            for j=1:(i-1)
                B=B+W(i,j)*(X(j,:)'*X(j,:));
            end
            for j=(i+1):N
                B=B+W(i,j)*(X(j,:)'*X(j,:));
            end
            lambda= max(eig(B));
            
            % calculate the position at which the minimum of the
            % majorization function is obtained
            z=(lambda-WR(i,i))*X(i,:)-(B*X(i,:)')'+WR(i,:)*X;
            X(i,:)=z/norm(z);
        
        end
        
        % gradient, obj value
        n=n+1;
        mygrad=P_tangent( X, dFw( X, R, W, sumW ) );
        normgrad=norm(mygrad,'fro');
        Fold=Fn;
        Fn=Fw(X,R,W,sumW);
        
        % output
        disp(sprintf('%d\t%e\t%e\t%s',n,normgrad,Fn));
        
    end
    
Xn=X;
